-- SQL code to create the persistency tables; postgresql version
--
-- This file is part of LIBPF
-- All rights reserved; do not distribute without permission.
-- (C) Copyright 2004-2024 Paolo Greppi simevo s.r.l.

DROP TABLE IF EXISTS I;
DROP TABLE IF EXISTS IVV;
DROP TABLE IF EXISTS IV;
DROP TABLE IF EXISTS Q;
DROP TABLE IF EXISTS QVV;
DROP TABLE IF EXISTS QV;
DROP TABLE IF EXISTS QMV;
DROP TABLE IF EXISTS QM;
DROP TABLE IF EXISTS S;
DROP TABLE IF EXISTS SVV;
DROP TABLE IF EXISTS SV;
DROP TABLE IF EXISTS N;

-- NODES
CREATE TABLE N (
        ID                      integer PRIMARY KEY,
        TAG                     character (50),
        DESCRIPTION             character (255),
        TYPE                    character (50),
        FULLTAG                 character (255),
        UUID                    character (36),
        CREATED_AT              timestamp WITH time zone,
        UPDATED_AT              timestamp WITH time zone,
        LOCKED_BY               character (50),
        LOCKED_UNTIL            timestamp WITH time zone,
        PARENT                  integer REFERENCES N (ID),
        ROOT                    integer REFERENCES N (ID),
        RANGE                   integer
);
CREATE INDEX n_parent_index ON n(parent);
-- INTEGERS
CREATE TABLE I (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255),
        VALUE                   integer
);
CREATE INDEX i_nid_index ON i(nid);
-- INTEGERVECTORS
CREATE TABLE IV (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255)
);
-- INTEGERVECTORVALUES
CREATE TABLE IVV (
        ID                      SERIAL PRIMARY KEY,
        VID                     integer REFERENCES IV (ID),
        VALUE                   integer
);
-- QUANTITIES
CREATE TABLE Q (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255),
        VALUE                   double precision,
        UNIT                    character (50),
        INPUT                   boolean,
        OUTPUT                  boolean
);
CREATE INDEX q_nid_index ON q(nid);
CREATE INDEX q_tag_index ON q(tag);
-- QUANTITYVECTORS
CREATE TABLE QV (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255),
        UNIT                    character (50),
        INPUT                   boolean,
        OUTPUT                  boolean
);
-- QUANTITYVECTORVALUES
CREATE TABLE QVV (
        ID                      SERIAL PRIMARY KEY,
        VID                     integer REFERENCES QV (ID),
        VALUE                   double precision
);
-- QUANTITYMATRICES
CREATE TABLE QM (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255),
        UNIT                    character (50),
        INPUT                   boolean,
        OUTPUT                  boolean,
        ROWS                    integer
);
-- QUANTITYMATRIXVALUES, arranged by columns
-- for example the matrix
-- | 0 1 |
-- | 2 3 |
-- will be stored as: [ 0 2 1 3 ]
CREATE TABLE QMV (
        ID                      SERIAL PRIMARY KEY,
        MID                     integer REFERENCES QM (ID),
        VALUE                   double precision
);
-- STRINGS
CREATE TABLE S (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255),
        VALUE                   character (255)
);
CREATE INDEX s_nid_index ON s(nid);
-- STRINGVECTORS
CREATE TABLE SV (
        ID                      SERIAL PRIMARY KEY,
        NID                     integer REFERENCES N (ID),
        TAG                     character (50),
        DESCRIPTION             character (255)
);
-- STRINGVECTORSVALUES
CREATE TABLE SVV (
        ID                      SERIAL PRIMARY KEY,
        VID                     integer REFERENCES SV (ID),
        VALUE                   character (255)
);
